#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function, unicode_literals

__license__   = 'GPL v3'
__copyright__ = u'2019, Jürgen Habelt <juergen@habelt-jena.de>'
__docformat__ = 'restructuredtext en'

load_translations()


# Wrap a stream so that output gets flushed immediately
# and also make sure that any unicode strings get safely
# encoded using "replace" before writing them.
class SafeUnbuffered:

    def __init__(self, stream):
        self.stream = stream
        self.encoding = stream.encoding
        if self.encoding == None:
            self.encoding = "utf-8"
    
    def write(self, data):
        if isinstance(data, unicode):
            data = data.encode(self.encoding,"replace")
        try:
            self.stream.write(data)
            self.stream.flush()
        except:
            # We can do nothing if a write fails
            pass
    
    def __getattr__(self, attr):
        return getattr(self.stream, attr)


import sys, os, re
from calibre.utils.config import config_dir

# The class that all Interface Action plug-in wrappers must inherit from
from calibre.customize import InterfaceActionBase
from calibre_plugins.define_content.json import JsonConfigWrapper
from calibre_plugins.define_content.logging import LoggingFactory
        

class DefineContentPlugin(InterfaceActionBase):
    '''
    This class is a simple wrapper that provides information about the actual
    plugin class. The actual interface plug-in class is called InterfacePlugin
    and is defined in the action.py file, as specified in the actual_plugin field
    below.

    The reason for having two classes is that it allows the command line
    calibre utilities to run without needing to load the GUI libraries.
    '''
    name                = "Define Content"
    description         = _(u"Define Content sets the user defined #content field using bookmark entries")
    supported_platforms = ['windows', 'linux']
    author              = u"Jürgen Habelt"
    version             = (1, 0, 1)
    minimum_calibre_version = (0, 7, 53)


    #: This field defines the GUI plug-in class that contains all the code that actually does something.
    actual_plugin       = 'calibre_plugins.define_content.action:DefineContentAction'

    def initialize(self):
        '''
        Initialize the Plug-in
        '''
        LoggingFactory().getLogger(self).info('Initializing Plug-in: Define Content')
        if JsonConfigWrapper.getInstance().version_changed(version = self.version):
            try:                
                LoggingFactory().getLogger(self).info('One time initialization after installation')
                self.prepare_scripts()
                JsonConfigWrapper.getInstance().version_changed(version = self.version, update = True)
                LoggingFactory().getLogger(self).info('One time initialization succeeded')

            except Exception as ex:
                LoggingFactory().getLogger(self).exception('Exception during one time initialization')

                

    def prepare_scripts(self):
        '''
        Prepares Ruby scripts for execution
        '''
        self.pluginsdir = os.path.join(config_dir, u"plugins")
        if not os.path.exists(self.pluginsdir):
            os.makedirs(self.pluginsdir)
        
        self.maindir = os.path.join(self.pluginsdir, u"DefineContent")
        if not os.path.exists(self.maindir):
            os.mkdir(self.maindir)
        
        dirs = ['scripts', 'scripts/templates', 'result']
        for entry in dirs:
            path = os.path.join(self.maindir, entry)
            if not os.path.exists(path):
                os.mkdir(path)
        
        scripts = self.load_resources(
            [
                'scripts/Bookmark.rb', 
                'scripts/EpubExtractor.rb', 
                'scripts/PdfExtractor.rb', 
                'scripts/HtmlGenerator.rb', 
                'scripts/Utils.rb', 
                'scripts/LoggerWrapper.rb', 
                'scripts/templates/Content.template'
            ])

        for entry, data in scripts.items():
            file_path = os.path.join(self.maindir, entry)
            
            try:
                os.remove(file_path)
            except:
                pass

            try:
                open(file_path, 'wb').write(data)
            except:
                pass


    def is_customizable(self):
        '''
        This method must return True to enable customization via
        Preferences->Plugins
        '''
        return True

    
    def can_be_disabled(self):
        '''
        Must return True to enable deactivation
        '''
        return True
    
    
    def config_widget(self):
        '''
        Implement this method and :meth:`save_settings` in your plug-in to
        use a custom configuration dialog.

        This method, if implemented, must return a QWidget. The widget can have
        an optional method validate() that takes no arguments and is called
        immediately after the user clicks OK. Changes are applied if and only
        if the method returns True.

        If for some reason you cannot perform the configuration at this time,
        return a tuple of two strings (message, details), these will be
        displayed as a warning dialog to the user and the process will be
        aborted.

        The base class implementation of this method raises NotImplementedError
        so by default no user configuration is possible.
        '''
        
        # It is important to put this import statement here rather than at the
        # top of the module as importing the config class will also cause the
        # GUI libraries to be loaded, which we do not want when using calibre
        # from the command line
        from calibre_plugins.define_content.config import ConfigWidget
        return ConfigWidget()

    
    def save_settings(self, config_widget):
        '''
        Save the settings specified by the user with config_widget.
        :param config_widget: The widget returned by :meth:`config_widget`.
        '''
        config_widget.save_settings()

        # Apply the changes
        if self.actual_plugin_:
            self.actual_plugin_.apply_settings()
